//	Zinc Application Framework - D_EVTMGR.CPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#include <z_evtmgr.hpp>
#include <z_sysevt.hpp>

// ----- ZafEventManager ---------------------------------------------------

int ZafEventManager::Get(ZafEventStruct &event, ZafQFlags flags)
{
	// Initialize the variables.
	ZafDevice *device;
	ZafQueueElement *element;
	int error = -1;

	// Stay in loop while no event conditions are met.
	do
	{
		// Call all the polled devices.
		if (PollDevices(flags))
		{
 			// Check for a user-device to do special message polling.
			for (device = First(); device; device = device->Next())
				device->Poll();
		}

		// Get the event.
		element = ReadFromEnd(flags) ? queueBlock.Last() : queueBlock.First();
		if (element)
		{
			event = element->event;
			if (DestroyEvent(flags))
				queueBlock.Subtract((ZafElement *)element);
			error = 0;		// Normal operation of the event manager.
		}
		else
		{
			event.type = S_REPAIR_DAMAGE;
			error = 0;
		}
/* START BLOCK COMMENT
**			else if (!Blocked(flags) || !First())
**				return (-2);	// No events were in queue.
END BLOCK COMMENT */
	} while (error);

	// Return the error status.
	return (error);
}


