//	Zinc Application Framework - D_FILDLG.HPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#if !defined(D_FILDLG_HPP)
#	define D_FILDLG_HPP
#	if !defined(Z_BNUM_HPP)
#		include<z_bnum.hpp>
#	endif
#	if !defined(Z_DATE_HPP)
#		include<z_date.hpp>
#	endif
#	if !defined(Z_TIME_HPP)
#		include<z_time.hpp>
#	endif
#	if !defined(Z_BUTTON_HPP)
#		include<z_button.hpp>
#	endif

// ----- Compiler specific files needed for the ZafFileDialog class -------

#if defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__)
#	include <dos.h>
#	include <dir.h>
#	include <direct.h>
#	include <errno.h>
#elif defined(__ZTC__)
#	include <dos.h>
#	include <cerror.h>
#	include <direct.h>
#	define _dos_getdrive dos_getdrive
#	define _dos_setdrive dos_setdrive
#elif defined(_MSC_VER)
#	include <dos.h>
#	if defined(DOS386)
#		include <pldos32.h>
#	endif
#	if defined(DOSX286)
#		define	_int86x	int86x
#		define	_int86	int86
#		define	_REGS	REGS
#	endif
#	include <direct.h>
#elif defined(__WATCOMC__)
#	include <io.h>
#	include <dos.h>
#	include <direct.h>
#elif defined(__HIGHC__) || defined(__IBMCPP__)
#	include <direct.h>
#endif

typedef unsigned DeviceClassification;
const DeviceClassification DEVICE_UNKNOWN			= 0x0000;
const DeviceClassification DEVICE_DRIVE			= 0x1000;
const DeviceClassification DEVICE_DRIVE_FIXED		= 0x1001;
const DeviceClassification DEVICE_DRIVE_REMOVABLE	= 0x1002;
const DeviceClassification DEVICE_DRIVE_REMOTE		= 0x1004;
const DeviceClassification DEVICE_DIRECTORY		= 0x2000;
const DeviceClassification DEVICE_FILE				= 0x4000;

// --------------------------------------------------------------------------
// ----- DeviceInfo ---------------------------------------------------------
// --------------------------------------------------------------------------

class ZAF_EXPORT DeviceInfo
{
	friend class ZAF_EXPORT ZafFileDialog;
public:
	DeviceClassification classification;
	ZafIChar name[32];
	ZafBignumData size;
	ZafDateData date;
	ZafTimeData time;

	int Inherited(DeviceClassification _classification)
		{ return (((classification & _classification) == _classification) ? true : false) ; }

private:
	int reserved;
#if defined(__BCPLUSPLUS__) | defined(__TCPLUSPLUS__)
	struct ffblk _reserved;
#elif defined(_MSC_VER) || defined(__WATCOMC__)
	struct _find_t _reserved;
#elif defined(__ZTC__)
	struct FIND *_reserved;
#endif
};

// --------------------------------------------------------------------------
// ----- ZafDirectoryItem ---------------------------------------------------
// --------------------------------------------------------------------------

class ZafDirectoryItem : public ZafButton
{
	friend class ZafFileDialog;
public:
	ZafDirectoryItem(int indentation, ZafIChar *pathName, ZafIChar *type);
	~ZafDirectoryItem(void);

	virtual ZafEventType Draw(const ZafEventStruct &event, ZafEventType ccode);

	static void ImageAllocate(void);
	static void ImageFree(bool globalRequest = false);

private:
	int indentation;

	static struct ZafItemSet
	{
		ZafIChar *name;
		ZafBitmapData *bitmapData;
		ZafUserFunction userFunction;
	} ZAF_FARDATA _item[];

	static ZafEventType DirectoryCallback(ZafWindowObject *object, ZafEventStruct &event, ZafEventType ccode);
	static ZafEventType DriveCallback(ZafWindowObject *object, ZafEventStruct &event, ZafEventType ccode);
	static ZafEventType FileCallback(ZafWindowObject *object, ZafEventStruct &event, ZafEventType ccode);
	static ZafEventType ParentDirectoryCallback(ZafWindowObject *object, ZafEventStruct &event, ZafEventType ccode);
};

#endif // D_FILDLG_HPP


